@@ -11,7 +11,7 @@ from process_tests import TestProcess
 from process_tests import wait_for_strings
 from redis import StrictRedis
 
-from redis_lock import Lock
+from redis_lock import Lock, AlreadyAcquired, NotAcquired
 from redis_lock import reset_all
 
 from conf import TIMEOUT
@@ -25,16 +25,22 @@ def redis_server(scope='module'):
         os.unlink(UDS_PATH)
     except OSError:
         pass
-    with TestProcess('redis-server', '--port', '0', '--unixsocket', UDS_PATH) as redis_server:
-        wait_for_strings(redis_server.read, TIMEOUT, "Running")
-        yield redis_server
+    with TestProcess('redis-server', '--port', '0', '--unixsocket', UDS_PATH) as process:
+        wait_for_strings(process.read, TIMEOUT, "Running")
+        yield process
+
+
+@pytest.fixture(scope='function')
+def conn(redis_server):
+    return StrictRedis(unix_socket_path=UDS_PATH)
 
 
 def test_simple(redis_server):
     with TestProcess(sys.executable, HELPER, 'test_simple') as proc:
         with dump_on_error(proc.read):
             name = 'lock:foobar'
-            wait_for_strings(proc.read, TIMEOUT,
+            wait_for_strings(
+                proc.read, TIMEOUT,
                 'Getting %r ...' % name,
                 'Got lock for %r.' % name,
                 'Releasing %r.' % name,
@@ -43,12 +49,13 @@ def test_simple(redis_server):
             )
 
 
-def test_no_block(redis_server):
-    with Lock(StrictRedis(unix_socket_path=UDS_PATH), "foobar"):
+def test_no_block(conn):
+    with Lock(conn, "foobar"):
         with TestProcess(sys.executable, HELPER, 'test_no_block') as proc:
             with dump_on_error(proc.read):
                 name = 'lock:foobar'
-                wait_for_strings(proc.read, TIMEOUT,
+                wait_for_strings(
+                    proc.read, TIMEOUT,
                     'Getting %r ...' % name,
                     'Failed to get %r.' % name,
                     'acquire=>False',
@@ -56,13 +63,13 @@ def test_no_block(redis_server):
                 )
 
 
-def test_expire(redis_server):
-    conn = StrictRedis(unix_socket_path=UDS_PATH)
+def test_expire(conn):
     with Lock(conn, "foobar", expire=TIMEOUT/4):
         with TestProcess(sys.executable, HELPER, 'test_expire') as proc:
             with dump_on_error(proc.read):
                 name = 'lock:foobar'
-                wait_for_strings(proc.read, TIMEOUT,
+                wait_for_strings(
+                    proc.read, TIMEOUT,
                     'Getting %r ...' % name,
                     'Got lock for %r.' % name,
                     'Releasing %r.' % name,
@@ -76,14 +83,15 @@ def test_expire(redis_server):
         lock.release()
 
 
-def test_double_acquire(redis_server):
-    lock = Lock(StrictRedis(unix_socket_path=UDS_PATH), "foobar")
+def test_double_acquire(conn):
+    lock = Lock(conn, "foobar")
     with lock:
         pytest.raises(RuntimeError, lock.acquire)
+        pytest.raises(AlreadyAcquired, lock.acquire)
 
 
-def test_plain(redis_server):
-    with Lock(StrictRedis(unix_socket_path=UDS_PATH), "foobar"):
+def test_plain(conn):
+    with Lock(conn, "foobar"):
         time.sleep(0.01)
 
 
@@ -98,6 +106,8 @@ def test_no_overlap(redis_server):
             wait_for_strings(proc.read, 10*TIMEOUT, 'DIED.')
 
             class Event(object):
+                pid = start = end = '?'
+
                 def __str__(self):
                     return "Event(%s; %r => %r)" % (self.pid, self.start, self.end)
 
@@ -127,8 +137,7 @@ def test_no_overlap(redis_server):
                             print("[%s/%s]" % (event, other))
                             raise
 
-def test_reset(redis_server):
-    conn = StrictRedis(unix_socket_path=UDS_PATH)
+def test_reset(conn):
     with Lock(conn, "foobar") as lock:
         lock.reset()
         new_lock = Lock(conn, "foobar")
@@ -136,8 +145,7 @@ def test_reset(redis_server):
         new_lock.release()
 
 
-def test_reset_all(redis_server):
-    conn = StrictRedis(unix_socket_path=UDS_PATH)
+def test_reset_all(conn):
     lock1 = Lock(conn, "foobar1")
     lock2 = Lock(conn, "foobar2")
     lock1.acquire(blocking=False)
@@ -151,8 +159,7 @@ def test_reset_all(redis_server):
     lock2.release()
 
 
-def test_owner_id(redis_server):
-    conn = StrictRedis(unix_socket_path=UDS_PATH)
+def test_owner_id(conn):
     unique_identifier = b"foobar-identifier"
     lock = Lock(conn, "foobar-tok", expire=TIMEOUT/4, id=unique_identifier)
     lock_id = lock.id
@@ -162,10 +169,22 @@ def test_owner_id(redis_server):
     lock.release()
 
 
-def test_token(redis_server):
-    conn = StrictRedis(unix_socket_path=UDS_PATH)
+def test_token(conn):
     lock = Lock(conn, "foobar-tok")
     tok = lock.id
     assert conn.get(lock._name) is None
     lock.acquire(blocking=False)
     assert conn.get(lock._name) == tok
+
+
+def test_bogus_release(conn):
+    lock = Lock(conn, "foobar-tok")
+    pytest.raises(NotAcquired, lock.release)
+    lock.release(force=True)
+
+def test_release_from_nonblocking_leaving_garbage(conn):
+    for _ in range(10):
+        lock = Lock(conn, 'release_from_nonblocking')
+        lock.acquire(blocking=False)
+        lock.release()
+        assert conn.llen('lock-signal:release_from_nonblocking') == 1
